"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const vscode = require("vscode");
const nls = require("vscode-nls");
const api_1 = require("../utils/api");
const languageModeIds_1 = require("../utils/languageModeIds");
const typeConverters = require("../utils/typeConverters");
const localize = nls.loadMessageBundle();
class FileReferencesCommand {
    constructor(client) {
        this.client = client;
        this.id = 'jtscript.findAllFileReferences';
    }
    execute(resource) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.client.apiVersion.lt(FileReferencesCommand.minVersion)) {
                vscode.window.showErrorMessage(localize('error.unsupportedVersion', "Find file references failed. Requires TypeScript 4.2+."));
                return;
            }
            if (!resource) {
                resource = (_a = vscode.window.activeTextEditor) === null || _a === void 0 ? void 0 : _a.document.uri;
            }
            if (!resource) {
                vscode.window.showErrorMessage(localize('error.noResource', "Find file references failed. No resource provided."));
                return;
            }
            const document = yield vscode.workspace.openTextDocument(resource);
            if (!languageModeIds_1.isSupportedLanguageMode(document)) {
                vscode.window.showErrorMessage(localize('error.unsupportedLanguage', "Find file references failed. Unsupported file type."));
                return;
            }
            const openedFiledPath = this.client.toOpenedFilePath(document);
            if (!openedFiledPath) {
                vscode.window.showErrorMessage(localize('error.unknownFile', "Find file references failed. Unknown file type."));
                return;
            }
            yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Window,
                title: localize('progress.title', "Finding file references")
            }, (_progress, token) => __awaiter(this, void 0, void 0, function* () {
                var _b;
                const response = yield this.client.execute('fileReferences', {
                    file: openedFiledPath
                }, token);
                if (response.type !== 'response' || !response.body) {
                    return;
                }
                const locations = response.body.refs.map(reference => typeConverters.Location.fromTextSpan(this.client.toResource(reference.file), reference));
                const config = vscode.workspace.getConfiguration('references');
                const existingSetting = config.inspect('preferredLocation');
                yield config.update('preferredLocation', 'view');
                try {
                    yield vscode.commands.executeCommand('editor.action.showReferences', resource, new vscode.Position(0, 0), locations);
                }
                finally {
                    yield config.update('preferredLocation', (_b = existingSetting === null || existingSetting === void 0 ? void 0 : existingSetting.workspaceFolderValue) !== null && _b !== void 0 ? _b : existingSetting === null || existingSetting === void 0 ? void 0 : existingSetting.workspaceValue);
                }
            }));
        });
    }
}
FileReferencesCommand.context = 'tsSupportsFileReferences';
FileReferencesCommand.minVersion = api_1.default.v420;
function register(client, commandManager) {
    function updateContext() {
        vscode.commands.executeCommand('setContext', FileReferencesCommand.context, client.apiVersion.gte(FileReferencesCommand.minVersion));
    }
    updateContext();
    commandManager.register(new FileReferencesCommand(client));
    return client.onTsServerStarted(() => updateContext());
}
exports.register = register;
