"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = exports.requireInlayHintsConfiguration = void 0;
const vscode = require("vscode");
const typescriptService_1 = require("../typescriptService");
const dependentRegistration_1 = require("../utils/dependentRegistration");
const typeConverters_1 = require("../utils/typeConverters");
const fileConfigurationManager_1 = require("./fileConfigurationManager");
const api_1 = require("../utils/api");
const dispose_1 = require("../utils/dispose");
var ExperimentalProto;
(function (ExperimentalProto) {
    let InlayHintKind;
    (function (InlayHintKind) {
        InlayHintKind["Type"] = "Type";
        InlayHintKind["Parameter"] = "Parameter";
        InlayHintKind["Enum"] = "Enum";
    })(InlayHintKind = ExperimentalProto.InlayHintKind || (ExperimentalProto.InlayHintKind = {}));
    (function (InlayHintKind) {
        function fromProtocolInlayHintKind(kind) {
            switch (kind) {
                case InlayHintKind.Parameter:
                    return vscode.InlayHintKind.Parameter;
                case InlayHintKind.Type:
                    return vscode.InlayHintKind.Type;
                case InlayHintKind.Enum:
                    return vscode.InlayHintKind.Other;
                default:
                    return vscode.InlayHintKind.Other;
            }
        }
        InlayHintKind.fromProtocolInlayHintKind = fromProtocolInlayHintKind;
    })(InlayHintKind = ExperimentalProto.InlayHintKind || (ExperimentalProto.InlayHintKind = {}));
})(ExperimentalProto || (ExperimentalProto = {}));
const inlayHintSettingNames = [
    fileConfigurationManager_1.InlayHintSettingNames.parameterNamesSuppressWhenArgumentMatchesName,
    fileConfigurationManager_1.InlayHintSettingNames.parameterNamesEnabled,
    fileConfigurationManager_1.InlayHintSettingNames.variableTypesEnabled,
    fileConfigurationManager_1.InlayHintSettingNames.propertyDeclarationTypesEnabled,
    fileConfigurationManager_1.InlayHintSettingNames.functionLikeReturnTypesEnabled,
    fileConfigurationManager_1.InlayHintSettingNames.enumMemberValuesEnabled,
];
class TypeScriptInlayHintsProvider extends dispose_1.Disposable {
    constructor(modeId, client, fileConfigurationManager) {
        super();
        this.client = client;
        this.fileConfigurationManager = fileConfigurationManager;
        this._onDidChangeInlayHints = new vscode.EventEmitter();
        this.onDidChangeInlayHints = this._onDidChangeInlayHints.event;
        this._register(vscode.workspace.onDidChangeConfiguration(e => {
            if (inlayHintSettingNames.some(settingName => e.affectsConfiguration(modeId + '.' + settingName))) {
                this._onDidChangeInlayHints.fire();
            }
        }));
    }
    provideInlayHints(model, range, token) {
        return __awaiter(this, void 0, void 0, function* () {
            const filepath = this.client.toOpenedFilePath(model);
            if (!filepath) {
                return [];
            }
            const start = model.offsetAt(range.start);
            const length = model.offsetAt(range.end) - start;
            yield this.fileConfigurationManager.ensureConfigurationForDocument(model, token);
            const response = yield this.client.execute('provideInlayHints', { file: filepath, start, length }, token);
            if (response.type !== 'response' || !response.success || !response.body) {
                return [];
            }
            return response.body.map(hint => {
                const result = new vscode.InlayHint(hint.text, typeConverters_1.Position.fromLocation(hint.position), hint.kind && ExperimentalProto.InlayHintKind.fromProtocolInlayHintKind(hint.kind));
                result.whitespaceBefore = hint.whitespaceBefore;
                result.whitespaceAfter = hint.whitespaceAfter;
                return result;
            });
        });
    }
}
TypeScriptInlayHintsProvider.minVersion = api_1.default.v440;
function requireInlayHintsConfiguration(language) {
    return new dependentRegistration_1.Condition(() => {
        const config = vscode.workspace.getConfiguration(language, null);
        const preferences = fileConfigurationManager_1.getInlayHintsPreferences(config);
        return preferences.includeInlayParameterNameHints === 'literals' ||
            preferences.includeInlayParameterNameHints === 'all' ||
            preferences.includeInlayEnumMemberValueHints ||
            preferences.includeInlayFunctionLikeReturnTypeHints ||
            preferences.includeInlayFunctionParameterTypeHints ||
            preferences.includeInlayPropertyDeclarationTypeHints ||
            preferences.includeInlayVariableTypeHints;
    }, vscode.workspace.onDidChangeConfiguration);
}
exports.requireInlayHintsConfiguration = requireInlayHintsConfiguration;
function register(selector, modeId, client, fileConfigurationManager) {
    return dependentRegistration_1.conditionalRegistration([
        requireInlayHintsConfiguration(modeId),
        dependentRegistration_1.requireMinVersion(client, TypeScriptInlayHintsProvider.minVersion),
        dependentRegistration_1.requireSomeCapability(client, typescriptService_1.ClientCapability.Semantic),
    ], () => {
        const provider = new TypeScriptInlayHintsProvider(modeId, client, fileConfigurationManager);
        return vscode.languages.registerInlayHintsProvider(selector.semantic, provider);
    });
}
exports.register = register;
