"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const jsonc = require("jsonc-parser");
const path = require("path");
const vscode = require("vscode");
const nls = require("vscode-nls");
const testUtils_1 = require("../test/testUtils");
const typescriptService_1 = require("../typescriptService");
const arrays_1 = require("../utils/arrays");
const dispose_1 = require("../utils/dispose");
const fs_1 = require("../utils/fs");
const languageDescription_1 = require("../utils/languageDescription");
const tsconfig_1 = require("../utils/tsconfig");
const tsconfigProvider_1 = require("./tsconfigProvider");
const localize = nls.loadMessageBundle();
var AutoDetect;
(function (AutoDetect) {
    AutoDetect["on"] = "on";
    AutoDetect["off"] = "off";
    AutoDetect["build"] = "build";
    AutoDetect["watch"] = "watch";
})(AutoDetect || (AutoDetect = {}));
/**
 * Provides tasks for building `tsconfig.json` files in a project.
 */
class TscTaskProvider extends dispose_1.Disposable {
    constructor(client) {
        super();
        this.client = client;
        this.projectInfoRequestTimeout = 2000;
        this.findConfigFilesTimeout = 5000;
        this.autoDetect = AutoDetect.on;
        this.tsconfigProvider = new tsconfigProvider_1.TsConfigProvider();
        this._register(vscode.workspace.onDidChangeConfiguration(this.onConfigurationChanged, this));
        this.onConfigurationChanged();
    }
    provideTasks(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const folders = vscode.workspace.workspaceFolders;
            if ((this.autoDetect === AutoDetect.off) || !folders || !folders.length) {
                return [];
            }
            const configPaths = new Set();
            const tasks = [];
            for (const project of yield this.getAllTsConfigs(token)) {
                if (!configPaths.has(project.fsPath)) {
                    configPaths.add(project.fsPath);
                    tasks.push(...(yield this.getTasksForProject(project)));
                }
            }
            return tasks;
        });
    }
    resolveTask(task) {
        return __awaiter(this, void 0, void 0, function* () {
            const definition = task.definition;
            if (/\\jtconfig.*\.json/.test(definition.jtconfig)) {
                // Warn that the task has the wrong slash type
                vscode.window.showWarningMessage(localize('badTsConfig', "JtScript Task in tasks.json contains \"\\\\\". JtScript tasks jtconfig must use \"/\""));
                return undefined;
            }
            const tsconfigPath = definition.jtconfig;
            if (!tsconfigPath) {
                return undefined;
            }
            if (task.scope === undefined || task.scope === vscode.TaskScope.Global || task.scope === vscode.TaskScope.Workspace) {
                // scope is required to be a WorkspaceFolder for resolveTask
                return undefined;
            }
            const tsconfigUri = task.scope.uri.with({ path: task.scope.uri.path + '/' + tsconfigPath });
            const tsconfig = {
                uri: tsconfigUri,
                fsPath: tsconfigUri.fsPath,
                posixPath: tsconfigUri.path,
                workspaceFolder: task.scope
            };
            return this.getTasksForProjectAndDefinition(tsconfig, definition);
        });
    }
    getAllTsConfigs(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const configs = arrays_1.flatten(yield Promise.all([
                this.getTsConfigForActiveFile(token),
                this.getTsConfigsInWorkspace(token),
            ]));
            return Promise.all(configs.map((config) => __awaiter(this, void 0, void 0, function* () { return (yield fs_1.exists(config.uri)) ? config : undefined; }))).then(arrays_1.coalesce);
        });
    }
    getTsConfigForActiveFile(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const editor = vscode.window.activeTextEditor;
            if (editor) {
                if (languageDescription_1.isTsConfigFileName(editor.document.fileName)) {
                    const uri = editor.document.uri;
                    return [{
                            uri,
                            fsPath: uri.fsPath,
                            posixPath: uri.path,
                            workspaceFolder: vscode.workspace.getWorkspaceFolder(uri)
                        }];
                }
            }
            const file = this.getActiveTypeScriptFile();
            if (!file) {
                return [];
            }
            const response = yield Promise.race([
                this.client.value.execute('projectInfo', { file, needFileNameList: false }, token),
                new Promise(resolve => setTimeout(() => resolve(typescriptService_1.ServerResponse.NoContent), this.projectInfoRequestTimeout))
            ]);
            if (response.type !== 'response' || !response.body) {
                return [];
            }
            const { configFileName } = response.body;
            if (configFileName && !tsconfig_1.isImplicitProjectConfigFile(configFileName)) {
                const normalizedConfigPath = path.normalize(configFileName);
                const uri = vscode.Uri.file(normalizedConfigPath);
                const folder = vscode.workspace.getWorkspaceFolder(uri);
                return [{
                        uri,
                        fsPath: normalizedConfigPath,
                        posixPath: uri.path,
                        workspaceFolder: folder
                    }];
            }
            return [];
        });
    }
    getTsConfigsInWorkspace(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const getConfigsTimeout = new vscode.CancellationTokenSource();
            token.onCancellationRequested(() => getConfigsTimeout.cancel());
            return Promise.race([
                this.tsconfigProvider.getConfigsForWorkspace(getConfigsTimeout.token).then(x => Array.from(x)),
                testUtils_1.wait(this.findConfigFilesTimeout).then(() => {
                    getConfigsTimeout.cancel();
                    return [];
                }),
            ]);
        });
    }
    static getCommand(project) {
        return __awaiter(this, void 0, void 0, function* () {
            if (project.workspaceFolder) {
                const localTsc = yield TscTaskProvider.getLocalTscAtPath(path.dirname(project.fsPath));
                if (localTsc) {
                    return localTsc;
                }
                const workspaceTsc = yield TscTaskProvider.getLocalTscAtPath(project.workspaceFolder.uri.fsPath);
                if (workspaceTsc) {
                    return workspaceTsc;
                }
            }
            // Use global tsc version
            return 'jtu';
        });
    }
    static getLocalTscAtPath(folderPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const platform = process.platform;
            const bin = path.join(folderPath, 'node_modules', '.bin');
            if (platform === 'win32' && (yield fs_1.exists(vscode.Uri.file(path.join(bin, 'jtu.cmd'))))) {
                return path.join(bin, 'jtu.cmd');
            }
            else if ((platform === 'linux' || platform === 'darwin') && (yield fs_1.exists(vscode.Uri.file(path.join(bin, 'jtu'))))) {
                return path.join(bin, 'jtu');
            }
            return undefined;
        });
    }
    getActiveTypeScriptFile() {
        const editor = vscode.window.activeTextEditor;
        if (editor) {
            const document = editor.document;
            if (document && (document.languageId === 'jtscript' || document.languageId === 'jtscriptreact')) {
                return this.client.value.toPath(document.uri);
            }
        }
        return undefined;
    }
    getBuildTask(workspaceFolder, label, command, args, buildTaskidentifier) {
        const buildTask = new vscode.Task(buildTaskidentifier, workspaceFolder || vscode.TaskScope.Workspace, localize('buildTscLabel', 'build - {0}', label), 'jtu', new vscode.ShellExecution(command, args), '$jtu');
        buildTask.group = vscode.TaskGroup.Build;
        buildTask.isBackground = false;
        return buildTask;
    }
    getWatchTask(workspaceFolder, label, command, args, watchTaskidentifier) {
        const watchTask = new vscode.Task(watchTaskidentifier, workspaceFolder || vscode.TaskScope.Workspace, localize('buildAndWatchTscLabel', 'watch - {0}', label), 'jtu', new vscode.ShellExecution(command, [...args, '--watch']), '$jtu-watch');
        watchTask.group = vscode.TaskGroup.Build;
        watchTask.isBackground = true;
        return watchTask;
    }
    getTasksForProject(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const command = yield TscTaskProvider.getCommand(project);
            const args = yield this.getBuildShellArgs(project);
            const label = this.getLabelForTasks(project);
            const tasks = [];
            if (this.autoDetect === AutoDetect.build || this.autoDetect === AutoDetect.on) {
                tasks.push(this.getBuildTask(project.workspaceFolder, label, command, args, { type: 'jtscript', jtconfig: label }));
            }
            if (this.autoDetect === AutoDetect.watch || this.autoDetect === AutoDetect.on) {
                tasks.push(this.getWatchTask(project.workspaceFolder, label, command, args, { type: 'jtscript', jtconfig: label, option: 'watch' }));
            }
            return tasks;
        });
    }
    getTasksForProjectAndDefinition(project, definition) {
        return __awaiter(this, void 0, void 0, function* () {
            const command = yield TscTaskProvider.getCommand(project);
            const args = yield this.getBuildShellArgs(project);
            const label = this.getLabelForTasks(project);
            let task;
            if (definition.option === undefined) {
                task = this.getBuildTask(project.workspaceFolder, label, command, args, definition);
            }
            else if (definition.option === 'watch') {
                task = this.getWatchTask(project.workspaceFolder, label, command, args, definition);
            }
            return task;
        });
    }
    getBuildShellArgs(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const defaultArgs = ['-p', project.fsPath];
            try {
                const bytes = yield vscode.workspace.fs.readFile(project.uri);
                const text = Buffer.from(bytes).toString('utf-8');
                const tsconfig = jsonc.parse(text);
                if (tsconfig === null || tsconfig === void 0 ? void 0 : tsconfig.references) {
                    return ['-b', project.fsPath];
                }
            }
            catch (_a) {
                // noops
            }
            return defaultArgs;
        });
    }
    getLabelForTasks(project) {
        if (project.workspaceFolder) {
            const workspaceNormalizedUri = vscode.Uri.file(path.normalize(project.workspaceFolder.uri.fsPath)); // Make sure the drive letter is lowercase
            return path.posix.relative(workspaceNormalizedUri.path, project.posixPath);
        }
        return project.posixPath;
    }
    onConfigurationChanged() {
        const type = vscode.workspace.getConfiguration('typescript.tsc').get('autoDetect');
        this.autoDetect = typeof type === 'undefined' ? AutoDetect.on : type;
    }
}
function register(lazyClient) {
    return vscode.tasks.registerTaskProvider('jtscript', new TscTaskProvider(lazyClient));
}
exports.register = register;
//# sourceMappingURL=taskProvider.js.map